package cz.zcu.kiv.vsp.msloup;

import java.util.Locale;

/**
 * Hlavn spotc tda Genertoru nhodnch sel s trojhelnkovm rozdlenm
 * @author Martin Sloup
 */
public class Main {

	/**
	 * Vstupn funkce programu
	 * @param args	argumenty programu
	 */
	public static void main(String[] args) {
		int count;
		double a;

		if (args.length == 2) {
			try {
				count = Integer.parseInt(args[0], 10);
				a = Double.parseDouble(args[1]);
			} catch (NumberFormatException e) {
				System.out.println("Spatny format vstupnich dat.");
				return;
			}

			printTriangleDistribution(count, a);
		} else {
			System.out.println("KIV/VSP: Generator trojuhelnikoveho rozdeleni");
			System.out.println("Autor: Martin Sloup, A08N0111P, msloup@students.zcu.cz");
			System.out.println();
			System.out.println("Vstup bez parametru! Bude generovano 100 000 hodnot pro a=1 a a=2");
			System.out.println();
			System.out.println("* Pro a = 1:");
			printTriangleDistribution(100000, 1);
			System.out.println();
			System.out.println("* Pro a = 2:");
			printTriangleDistribution(100000, 2);
		}

	}

	/**
	 * Vytiskne stedn hodnotu a rozptyl pro matematick vpoet i vpoet z generovanch hodnot. Nakonec vype histogram.
	 * @param count poet generovanch hodnot
	 * @param a	sted trojhelnku
	 */
	private static void printTriangleDistribution(int count, double a) {
		Distribution dist = new TriangularDistribution(count, a);

		System.out.println("E_teorie=" + dist.getE());
		System.out.println("D_teorie=" + dist.getD());

		dist.iterate();

		System.out.println("E_vypocet=" + dist.getComputedE());
		System.out.println("D_vypocet=" + dist.getComputedD());

		System.out.println();

		double[] histogram = dist.getHistogram();

		//vypis histogramu
		for (int i = 0; i < histogram.length; i++)
			System.out.println(String.format(Locale.US, "%.4f:%s", i * dist.getHistogramItemSize() + dist.getHistogramMin(),RepeatChar('*', (int) Math.floor(histogram[i] * 100))));
	}

	/**
	 * Vrac [count] poet znak [ch]
	 * @param ch	znak
	 * @param count	poet opakovn znaku
	 * @return		text
	 */
	public static String RepeatChar(char ch, int count) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < count; i++)
			sb.append(ch);
		return sb.toString();
	}

}
