package cz.zcu.kiv.vsp.msloup;

import java.util.Random;

public class TriangularDistribution implements Distribution {
	private int count;
	private double a;

	private double Sx = 0;
	private double Sx2 = 0;

	private int histogramCount = 0;
	private double histogramMin = -0.5;

	private Random random;
	private double[] histogram;
	private double histogramItemSize;

	/**
	 * Konstruktor pro trojhelnkov rozdlen
	 * @param count	poet generovanch hodnot
	 * @param a	sted trojhelnku
	 */
	public TriangularDistribution(int count, double a) {
		this.count = count;
		this.a = a;
		histogramCount = 16; //(int) (2*a / 0.5) + 2;

		random = new Random();

		histogram = new double[histogramCount + 2];
		histogramItemSize = Math.abs(getHistogramMax()) / histogramCount;
		histogramMin = -histogramItemSize;
	}

	@Override
	public double getComputedD() {
		return (Sx2 / count) - getComputedE() * getComputedE();
	}

	@Override
	public double getComputedE() {
		return Sx / count;
	}

	@Override
	public double getD() {
		return (a*a)/6.;
	}

	@Override
	public double getE() {
		return a;
	}

	@Override
	public double[] getHistogram() {
		double[] ret = histogram.clone();

		for(int i = 0; i < ret.length; i++)
			ret[i] = ret[i] / count;

		return ret;
	}

	@Override
	public double getHistogramMin() {
		return histogramMin;
	}

	@Override
	public double getHistogramItemSize() {
		return histogramItemSize;
	}

	@Override
	public double getHistogramMax() {
		return 2*a;
	}

	@Override
	public void iterate() {
		// TODO Auto-generated method stub
		for (int i = 0; i < count; i++) {
			double value = generateNumber();

			//pocitani hodnot pro stredni hodnotu a rozptyl
			Sx+= value;
			Sx2+= value * value;

			//pridani hodnoty do histogramu
			int index = (int) ((value - getHistogramMin()) / getHistogramItemSize());
			histogram[index]++;
		}
	}

	/**
	 * Vrac vygenerovan slo v trojuhelnkovm rozdlen
	 * @return	vygenerovan slo
	 */
	private double generateNumber() {
		double x = random.nextDouble();

		if (x <= 1./2) {
			return Math.sqrt(2. * x * a * a );
		} else {
			return 2.*a - Math.sqrt((1.-x)*2.*a*a);
		}
	}
}
